# https://github.com/hroncok/license/blob/master/license/licenses.py
# https://github.com/spdx/license-list-data/blob/master/json/licenses.json

from ._licenses import Licenses
from ._license import License


licenses = Licenses()

__all__ = ['Licenses', 'License', 'licenses']





# https://github.com/bottlepy/bottle/commit/fa7733e075da0d790d809aa3d2f53071897e6f76
# https://github.com/pydanny/cached-property/blob/master/cached_property.py
class cached_property(object):  # noqa: N801
    """
    A property that is only computed once per instance and then replaces itself
    with an ordinary attribute. Deleting the attribute resets the property.
    """
    def __init__(self, func):
        self.__doc__ = func.__doc__
        self.func = func

    def __get__(self, obj, cls):
        if obj is None:
            return self
        value = obj.__dict__[self.func.__name__] = self.func(obj)
        return value




# -*- coding: utf-8 -*-

# DO NOT EDIT THIS FILE!
# This file has been autogenerated by dephell <3
# https://github.com/dephell/dephell

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


import os.path

readme = ''
here = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(here, 'README.rst'), 'rb') as stream:
    readme = stream.read().decode('utf8')


setup(
    long_description=readme,
    name='dephell-licenses',
    version='0.1.4',
    description='Get info about OSS licenses',
    python_requires='>=3.5',
    project_urls={'repository': 'https://github.com/dephell/dephell_licenses'},
    author='Gram',
    author_email='master_fess@mail.ru',
    license='MIT',
    keywords='dephell packaging licenses pypi',
    classifiers=['Development Status :: 4 - Beta', 'Intended Audience :: Developers', 'License :: OSI Approved :: MIT License', 'Programming Language :: Python', 'Topic :: Software Development :: Libraries :: Python Modules', 'Programming Language :: Python :: 3.5', 'Programming Language :: Python :: 3.6', 'Programming Language :: Python :: 3.7', 'Programming Language :: Python :: 3.8', 'Programming Language :: Python :: Implementation :: CPython', 'Programming Language :: Python :: Implementation :: PyPy', 'Operating System :: MacOS', 'Operating System :: Microsoft :: Windows', 'Operating System :: Unix'],
    packages=['dephell_licenses'],
    package_data={'dephell_licenses': ['*.json']},
    install_requires=['attrs', 'requests'],
)



import re
import textwrap
from typing import Dict, Any, List

import attr
import requests

from ._cached_property import cached_property


# <<var;name="copyright";original="<year> <copyright holders>";match=".+">>
# <<var;name=organizationClause3;original=the copyright holder;match=.+>>
rex_var = re.compile(
    r'<<var;'
    r'name="?(?P<name>[a-zA-Z0-9]+)"?;'
    r'original="?(?P<original>.+?)"?;'
    r'match="?(?P<match>.+?)"?>>'   # noQA: C812
)
rex_junk = re.compile(r'<<.+?>>')


@attr.s()
class License:
    id = attr.ib(type=str)
    classifier = attr.ib(type=str)
    name = attr.ib(type=str)

    approved = attr.ib(type=bool)
    deprecated = attr.ib(type=bool)

    links = attr.ib(type=List[str])
    url = attr.ib(type=str)

    @cached_property
    def _details(self) -> Dict[str, Any]:
        return requests.get(self.url).json()

    @property
    def template(self) -> str:
        return self._details['standardLicenseTemplate']

    @property
    def comments(self) -> str:
        return self._details['licenseComments']

    @cached_property
    def vars(self) -> List[Dict[str, str]]:
        vars = []
        for match in rex_var.finditer(self.template):
            var = match.groupdict()
            var['text'] = match.group(0)
            vars.append(var)
        return vars

    def make_text(self, *, wrap: bool = True, **kwargs) -> str:
        text = self.template
        for var in self.vars:
            value = kwargs.get(var['name'], var['original'])
            text = text.replace(var['text'], value)
        text = rex_junk.sub('', text).strip()

        if wrap:
            wrapper = textwrap.TextWrapper(
                width=78,
                break_long_words=False,
                replace_whitespace=False,
            )
            text = '\n'.join(wrapper.fill(line) for line in text.splitlines())

        while '\n\n\n' in text:
            text = text.replace('\n\n\n', '\n\n')

        return text + '\n'



import json
import re
from pathlib import Path
from typing import Tuple, Dict, Iterator, Optional

from ._cached_property import cached_property
from ._license import License


rex_clean = re.compile(r'[^a-zA-Z0-9]+')


class Licenses:
    @cached_property
    def all(self) -> Tuple[License, ...]:
        path = Path(__file__).parent / 'licenses.json'
        licenses = json.loads(path.read_text(encoding='utf-8'))
        return tuple(License(**license) for license in reversed(licenses))

    # indices

    @cached_property
    def _reverse_index_classifier(self) -> Dict[str, License]:
        return {license.classifier: license for license in self.all}

    @cached_property
    def _reverse_index_id(self) -> Dict[str, License]:
        return {license.id: license for license in self.all}

    @cached_property
    def _reverse_index_cleaned_id(self) -> Dict[str, License]:
        return {self._clean_id(license.id): license for license in self.all}

    # getters

    def get_by_classifier(self, classifier: str) -> Optional[License]:
        return self._reverse_index_classifier.get(classifier)

    def get_by_id(self, license_id: str) -> Optional[License]:
        license = self._reverse_index_id.get(license_id)
        if license is not None:
            return license
        return self._reverse_index_cleaned_id.get(self._clean_id(license_id))

    def get_by_name(self, name: str) -> Optional[License]:
        for license in self.all:
            if license.name == name:
                return license
        for license in self.all:
            if license.classifier.split(' :: ')[-1] == name:
                return license
        return None

    # private methods

    @staticmethod
    def _clean_id(license_id: str) -> str:
        return rex_clean.sub('', license_id).replace('v', '')

    # magic methods

    def __iter__(self) -> Iterator[License]:
        yield from self.all

    def __repr__(self) -> str:
        return '{}()'.format(type(self).__name__)
